; Countdown timer demo #1
; Charles Wieland, 8/5/2014
; junkosoft.org

      processor 6502
      include vcs.h
	  
; Constant values
VBLANK_TIME = $2B
OSCAN_TIME = $22
LOGO_HEIGHT = $1A
SHORT_DELAY = $0A
PRE_DELAY = $78
POST_DELAY = $78
TIMER_LENGTH = $96

; Labels for RAM	  
	SEG.U VARS
	ORG $80

LOGOCOLOR ds 1
LOGOSCANLINE ds 1
LOGOSCANSTART ds 1
LOGOSCANEND ds 1
LOGOTEMPGRPVALUE ds 1
BACKGROUNDCOLOR ds 1
FRAMEDELAY ds 1
TENSDIGITPOINTER ds 2
ONESDIGITPOINTER ds 2
TIMERFRAME ds 1
TIMERSEC ds 1

; Declare program section
    SEG CODE  	  
	ORG $F000
PositionObject:
	STA HMCLR
	SEC
	STA WSYNC

WaitForPosition:
	SBC #$0F
	BCS WaitForPosition

	EOR #$07
	ASL
	ASL
	ASL
	ASL
	STA RESP0,X
	STA HMP0,X
	STA WSYNC
	STA HMOVE
	RTS
	
Start	
; Initialize the system
	SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

ClearStack
	STA $00,X
	DEX
	BNE ClearStack

; Set up the initial values
	JSR InitializeValues

DrawVerticalSync
	LDA #$82
	STA VBLANK

	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA WSYNC

	LDA #$00
	STA VSYNC

	LDA #$02
	STA VBLANK
	
; Start vertical blank	
DrawVBlank
	LDA #VBLANK_TIME
	STA TIM64T

; Set large size sprite for number display
	LDA #$07
	STA NUSIZ0
	STA NUSIZ1

; Check the reset button
	LDA SWCHB
	LSR
	BCS ProcessTimer
	
; Reset all values
	JSR InitializeValues

ProcessTimer
	LDA TIMERSEC
	BEQ WaitForVBlankTimer
	
; Decrement the timer frame count
	DEC TIMERFRAME
	BNE CalculateDigitPointers

; Decrement the timer second count
	SED
	SEC
	SBC #$01
	STA TIMERSEC
	CLD

; Reset the timer frame count
	LDA #$3C
	STA TIMERFRAME
	
; Process the timer frame count
CalculateDigitPointers
	LDA TIMERSEC

; Calculate the digit memory pointers
	JSR CalculateScoreDigitPointer
	STY ONESDIGITPOINTER
	STX TENSDIGITPOINTER
	
ProcessLogoTimer
; Decrease the delay timer
	DEC FRAMEDELAY
	BNE SetPlayerColors
	
	DEC LOGOSCANSTART
	DEC LOGOSCANEND
	BMI ResetTop
	BEQ SetPostDelay
	
	LDA #SHORT_DELAY
	STA FRAMEDELAY
	JMP SetPlayerColors
	
; Reset to top of logo
ResetTop
	JSR ResetScanPosition
	JMP SetPlayerColors
	
SetPostDelay
	LDA #POST_DELAY
	STA FRAMEDELAY
	
SetPlayerColors
	LDA LOGOCOLOR
	STA COLUP0
	STA COLUP1
	
WaitForVBlankTimer
	LDA INTIM
	BNE WaitForVBlankTimer
	STA WSYNC
    STA VBLANK
	
; Position the tens digit (3 scanlines)
	LDA #$38
	LDX #$00
	JSR PositionObject
	STA WSYNC

; Position the ones digit (3 scanlines)
	LDA #$60
	LDX #$01
	JSR PositionObject
	STA WSYNC
	
	LDY BACKGROUNDCOLOR
	STY COLUBK
	
; Draw 54 empty lines
	LDY #$36
DrawTop
	STA WSYNC
	DEY
	BNE DrawTop
	LDY #$07
	
DrawNumbers
	LDX #$08
	LDA (TENSDIGITPOINTER),Y
	STA GRP0
	LDA (ONESDIGITPOINTER),Y
	STA GRP1
	
DrawLine
	STA WSYNC
	DEX
	BNE DrawLine
	
	DEY
	BPL DrawNumbers
	
; Clear out player data
	STX GRP0
	STX GRP1
	
; Draw 54 empty lines
	LDY #$36
DrawBottom
	STA WSYNC
	DEY
	BNE DrawBottom
	
	LDA #$00
	STA NUSIZ0
	STA NUSIZ1

; Prepare for the logo (2 scanlines)
DrawLogo
	STA WSYNC
	LDA #$00
	STA COLUBK
	STA WSYNC
	STA GRP0 
	STA GRP1 
	LDY LOGOSCANSTART
	STY LOGOSCANLINE  
	LDA #$03
	STA NUSIZ0  
	STA NUSIZ1
	STA VDELP0  
	STA VDELP1
	LDA #$F0 
	STA HMP0
	STA RESP0
	STA RESP1
	STA WSYNC
	STA HMOVE

; Draw logo (12 scanlines)
DrawLogoScanline
	LDY LOGOSCANLINE
	LDA logodata1,Y 
	STA GRP0 
	STA WSYNC
	LDA logodata2,Y 
	STA GRP1
	LDA logodata3,Y 
	STA GRP0 
	LDA logodata4,Y 
	STA LOGOTEMPGRPVALUE  
	LDA logodata5,Y 
	TAX
	LDA logodata6,Y 
	TAY
	LDA LOGOTEMPGRPVALUE  
	STA GRP1
	STX GRP0
	STY GRP1
	STA GRP0
	DEC LOGOSCANLINE
	LDA LOGOSCANLINE
	CMP LOGOSCANEND
	BPL DrawLogoScanline
	STA WSYNC

	LDA #$00 
	STA GRP0
	STA GRP1
	STA VDELP0  
	STA VDELP1  
	STA NUSIZ0  
	STA NUSIZ1  
	
	LDA #$02
	STA VBLANK
	
; Set timer for overscan
	LDA #OSCAN_TIME
	STA TIM64T
		
	LDA #$02
	STA VBLANK
	
; DrawOverscan (30 scanlines)
WaitForOverscan
	LDA INTIM
	BNE WaitForOverscan
	JMP DrawVerticalSync
	
logodata1
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %11101110 ; |### ### |
	.byte %10001010 ; |#   # # |
	.byte %10001010 ; |#   # # |
	.byte %10001110 ; |#   ### |
	.byte %10000000 ; |#       |
	.byte %11100000 ; |###     |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00011111 ; |   #####|
 	.byte %00011111 ; |   #####|
	.byte %00000000 ; |        |
	.byte %00011110 ; |   #### |
	.byte %00010010 ; |   #  # |
	.byte %00000010 ; |      # |
	.byte %00000010 ; |      # |
	.byte %00000010 ; |      # |
	.byte %00000010 ; |      # |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |


logodata2
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %10001110 ; |#   ### |
	.byte %10000010 ; |#     # |
	.byte %11101110 ; |### ### |
	.byte %10101010 ; |# # # # |
	.byte %10101010 ; |# # # # |
	.byte %11101010 ; |### # # |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %00000000 ; |        |
	.byte %11101010 ; |### # # |
	.byte %10101010 ; |# # # # |
	.byte %10101010 ; |# # # # |
	.byte %10100110 ; |# #  ## |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |

logodata3
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000111 ; |     ###|
	.byte %00000001 ; |       #|
	.byte %10010111 ; |#  # ###|
	.byte %10010101 ; |#  # # #|
	.byte %10010101 ; |#  # # #|
	.byte %11000111 ; |##   ###|
	.byte %00010000 ; |   #    |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %00000000 ; |        |
	.byte %10101110 ; |# # ### |
	.byte %11001010 ; |##  # # |
	.byte %10101010 ; |# # # # |
	.byte %10001110 ; |#   ### |
	.byte %10000000 ; |#       |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |

logodata4
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %01010010 ; | # #  # |
	.byte %01010010 ; | # #  # |
	.byte %01010010 ; | # #  # |
	.byte %01110111 ; | ### ###|
	.byte %01000010 ; | #    # |
	.byte %01000010 ; | #    # |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %11111110 ; |####### |
	.byte %11111111 ; |########|
	.byte %00011111 ; |   #####|
 	.byte %00001111 ; |    ####|
	.byte %00011111 ; |   #####|
	.byte %00111110 ; |  ##### |
	.byte %01111100 ; | #####  |
	.byte %11111000 ; |#####   |
	.byte %11111100 ; |######  |
	.byte %01111111 ; | #######|
	.byte %00111111 ; |  ######|

logodata5
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00011101 ; |   ### #|
	.byte %00010001 ; |   #   #|
	.byte %00011101 ; |   ### #|
	.byte %00000101 ; |     # #|
	.byte %00011101 ; |   ### #|
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %10000000 ; |#       |
	.byte %10111010 ; |# ### # |
	.byte %00101011 ; |  # # ##|
	.byte %00101010 ; |  # # # |
	.byte %00111011 ; |  ### ##|
 	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

logodata6
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %11010001 ; |## #   #|
	.byte %01010001 ; | # #   #|
	.byte %01010111 ; | # # ###|
	.byte %01010101 ; | # # # #|
	.byte %11010101 ; |## # # #|
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
	.byte %00000000 ; |        |
 	.byte %00010000 ; |   #    |
	.byte %00010000 ; |   #    |
	.byte %00010000 ; |   #    |
	.byte %10111000 ; |# ###   |
	.byte %00010000 ; |   #    |
	.byte %00000000 ; |        |
	.byte %11111000 ; |#####   |
	.byte %11111000 ; |#####   |
	
ResetScanPosition
	SEC
	LDA #LOGO_HEIGHT
	STA LOGOSCANSTART
	SBC #$0C
	STA LOGOSCANEND
	LDA #PRE_DELAY
	STA FRAMEDELAY
	CLC
	LDA LOGOCOLOR
	ADC #$10
	STA LOGOCOLOR
	SBC #$07
	STA BACKGROUNDCOLOR
	RTS
	
InitializeValues
; Set the score color
	LDA #$FE
	STA LOGOCOLOR
	LDA #$3C
	STA TIMERFRAME
	LDA #TIMER_LENGTH
	STA TIMERSEC
	LDA #$FF
	LDX #$01
	STA TENSDIGITPOINTER,X
	STA ONESDIGITPOINTER,X
	JSR ResetScanPosition
	RTS
	
; Gets the low byte of the pointer for each digit
CalculateScoreDigitPointer:
	TAX
	AND #$0F 
	ASL
	ASL
	ASL
	TAY
	TXA
	AND #$F0 
	LSR
	TAX
	RTS
	
	org $FF00
numbers
	; Zero
	.byte %11111111 ; | ###### |
	.byte %11111111 ; |###  ###|
	.byte %11000011 ; |##    ##|
	.byte %11000011 ; |##    ##|
	.byte %11000011 ; |##    ##|
	.byte %11000011 ; |##    ##|
	.byte %11111111 ; |###  ###|
	.byte %11111111 ; | ###### |

	; One
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |
	.byte %00011000 ; |   ##   |

	; Two
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000000 ; |##      |
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %00000011 ; |      ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

	; Three
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %00000011 ; |      ##|
	.byte %00111111 ; |  ######|
	.byte %00111111 ; |  ######|
	.byte %00000011 ; |      ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

	; Four
	.byte %00000011 ; |      ##|
	.byte %00000011 ; |      ##|
	.byte %00000011 ; |      ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000011 ; |##    ##|
	.byte %11000011 ; |##    ##|
	.byte %11000011 ; |##    ##|

	; Five
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %00000011 ; |      ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000000 ; |##      |
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

	; Six
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000011 ; |##    ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000000 ; |##      |
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

	; Seven
	.byte %00000011 ; |      ##|
	.byte %00000011 ; |      ##|
	.byte %00000011 ; |      ##|
	.byte %00000011 ; |      ##|
	.byte %00000011 ; |      ##|
	.byte %00000011 ; |      ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

	; Eight
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000011 ; |##    ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000011 ; |##    ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

	; Nine
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %00000011 ; |      ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000011 ; |##    ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

	; A
	.byte %11000011 ; |##    ##|
	.byte %11000011 ; |##    ##|
	.byte %11000011 ; |##    ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000011 ; |##    ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

	; B
	.byte %11111110 ; |####### |
	.byte %11111111 ; |########|
	.byte %11000011 ; |##    ##|
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000011 ; |##    ##|
	.byte %11111111 ; |########|
	.byte %11111110 ; |####### |

	; C
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000000 ; |##      |
	.byte %11000000 ; |##      |
	.byte %11000000 ; |##      |
	.byte %11000000 ; |##      |
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

	; D
	.byte %11111110 ; |####### |
	.byte %11111111 ; |########|
	.byte %11000011 ; |##    ##|
	.byte %11000011 ; |##    ##|
	.byte %11000011 ; |##    ##|
	.byte %11000011 ; |##    ##|
	.byte %11111111 ; |########|
	.byte %11111110 ; |####### |

	; E
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	.byte %11000000 ; |##      |
	.byte %11111100 ; |######  |
	.byte %11111100 ; |######  |
	.byte %11000000 ; |##      |
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|

	; F
	.byte %11000000 ; |##      |
	.byte %11000000 ; |##      |
	.byte %11000000 ; |##      |
	.byte %11111100 ; |######  |
	.byte %11111100 ; |######  |
	.byte %11000000 ; |##      |
	.byte %11111111 ; |########|
	.byte %11111111 ; |########|
	
	ORG $FFFC	   
	.word Start
	.word Start